use std::{default::Default, ffi::CString};

use nix::{
    libc::{gid_t, uid_t},
    sched::CloneFlags,
    sys::signal::{Signal, SIGKILL},
};

use crate::unshare::{GidMap, UidMap};

type IdMaps = (Vec<UidMap>, Vec<GidMap>);

pub struct Config {
    pub death_sig: Option<Signal>,
    pub work_dir: Option<CString>,
    pub uid: Option<uid_t>,
    pub gid: Option<gid_t>,
    pub id_maps: Option<IdMaps>,
    pub namespaces: CloneFlags,
    pub restore_sigmask: bool,
    pub make_group_leader: bool,
    pub drop_caps: Option<Vec<nix::libc::c_ulong>>,
}

impl Default for Config {
    fn default() -> Config {
        Config {
            death_sig: Some(SIGKILL),
            work_dir: None,
            uid: None,
            gid: None,
            id_maps: None,
            namespaces: CloneFlags::empty(),
            restore_sigmask: true,
            make_group_leader: false,
            drop_caps: None,
        }
    }
}
